_AUTO_RELOAD_DEBUG = true 

local instrument_looper = nil
local instrument_looper_index = 0
local sample_destination = nil
local buffer_destination = nil
local parameter_start = nil
local parameter_end = nil

local bool_bad_setup = true

function init()

    buffer_destination = nil
  
    sample_destination = nil
	
	instrument_looper = nil
    instrument_looper_index = 0
	
	bool_bad_setup = true

end

function set_loop_start(int_parameter)

  int_parameter = math.min(126, int_parameter)

  if int_parameter >= parameter_end.value then
    parameter_end.value = int_parameter + 1
  end

  if buffer_destination then
 
	if not buffer_destination.has_sample_data then
	  print("Sample destination has no audio data")
	  return
	end
	
	if sample_destination.loop_mode == renoise.Sample.LOOP_MODE_OFF then
	  sample_destination.loop_mode = renoise.Sample.LOOP_MODE_FORWARD
	end
	
	sample_destination.loop_start = 1 + buffer_destination.number_of_frames * int_parameter / 128
	
  end
  
end

function set_loop_end(int_parameter)

  int_parameter = math.max(1, int_parameter)

  if int_parameter <= parameter_start.value then
    parameter_start.value = int_parameter - 1
  end

  if buffer_destination then
 
	if not buffer_destination.has_sample_data then
	  print("Sample destination has no audio data")
	  return
	end
	
	if sample_destination.loop_mode == renoise.Sample.LOOP_MODE_OFF then
	  sample_destination.loop_mode = renoise.Sample.LOOP_MODE_FORWARD
	end
	
	sample_destination.loop_end = buffer_destination.number_of_frames * int_parameter / 127 - 1
	
  end
  
end

function look_for_looper_renaming()
  look_for_looper()
end

function look_for_looper()

  local int_cont = 0
  local int_instruments = table.getn(renoise.song().instruments)

  bool_bad_setup = true

  for instrument = 1, int_instruments do 
  
    if bool_bad_setup then

      int_cont = int_cont + 1
	  local instrument = renoise.song().instruments[int_cont]

	  if not instrument.name_observable:has_notifier(look_for_looper_renaming) then
        instrument.name_observable:add_notifier(look_for_looper_renaming)
	    --print(int_cont .. " hadn't got it, now it has got it")
	  else
  	    --print(int_cont .. " already has got it")
      end
	
	  if instrument.name == "Looper" then
	
	    local int_samples = table.getn(renoise.song().instruments[int_cont].samples)
	  
	    if int_samples < 1 then
	  
	      print("not enough samples in the instrument '" .. instrument.name .. "'")
	      bool_bad_setup = true -- setup is incomplete and nothing can ben done
		
  	    else 
	  
	      for int_sample = 1, int_samples do
		
		    local sample = instrument.samples[int_sample]
			
			if not sample.name_observable:has_notifier(look_for_looper_renaming) then
			  sample.name_observable:add_notifier(look_for_looper_renaming)
			end
			
		    if buffer_destination == nil and sample.name == "Looper destination" then
			
			  print("Got destination")
			  sample_destination = sample
		      buffer_destination = sample.sample_buffer
			  
		    end
		
		  end
		
		  
		  local track_master = get_master_track()
			
		  if track_master then
			
			print("Master track found")
			
			local bool_midiccdevice_found = false
			local device = nil
			
			for int_device = 1, table.getn(track_master.devices) do
			  
			  if not bool_midiccdevice_found then
				
			    device = track_master.devices[int_device]
				bool_midiccdevice_found = device.name == "*Instr. MIDI Control"
				
			  end
			  
			end
			  
			if not bool_midiccdevice_found then
			  
			  print("Fake MidiCcDevice created")			
			  device =  track_master:insert_device_at("Audio/Effects/Native/*Instr. MIDI Control",2)
			  bool_bad_setup = false
			    
			else
			  
			  print("Fake MidiCcDevice found")
			  bool_bad_setup = false
			  
			end
			  
			parameter_start = device.parameters[1]
			parameter_end = device.parameters[2]
			
			parameter_start.value_observable:add_notifier(
			  function()
			     set_loop_start(parameter_start.value)
			  end
			)

			parameter_end.value_observable:add_notifier(
			  function()
			     set_loop_end(parameter_end.value)
			  end
			)
			 
  		  end
			  
		  if not bool_bad_setup then
		  
		    renoise.app():show_status("Looper setup completed successfully for instrument " .. int_cont - 1 .. " - '" .. instrument.name .. "'")
			instrument_looper = instrument
			instrument_looper_index = int_cont 
			
		  end
	  
	    end

	  end
  
    end

  end
	
end

renoise.tool().app_new_document_observable:add_notifier(
  function()
  
	print("new document")

	init()  
	
    if renoise.song() then
	
	  look_for_looper()

      if renoise.song().instruments_observable:has_notifier(look_for_looper) then
        renoise.song().instruments_observable:remove_notifier(look_for_looper)
      end
  
      renoise.song().instruments_observable:add_notifier(look_for_looper)
  
    end
	
  end
)

renoise.tool().app_release_document_observable:add_notifier(
  function()

    init()  

  end
)

function get_master_track()

  local track_master = nil
	
	--go to master track and add a MidiCCDevice there, if not already present
  for int_track = 1, table.getn(renoise.song().tracks) do
	
	if track_master == nil then
	  
	  local track = renoise.song().tracks[int_track]
	  if track.type == renoise.Track.TRACK_TYPE_MASTER then
		
        track_master = track		
		  
	  end
		
	end
	
  end
	
  return track_master
  
end